# Copyright (c) 2015 Dave Love, Liverpool University
# MIT licence, per Fedora policy

# Possible fixme: The headers and libraries

Name:		feast
Version:	3.0
Release:	1%{?dist}
Summary:	Eigenvalue/eigenvector solver

License:	BSD
URL:		http://www.ecs.umass.edu/~polizzi/feast
# not directly downloadable from http://www.ecs.umass.edu/~polizzi/feast/download.htm
Source0:	feast_%version.tgz
# build shared libs
Patch1:		feast-shared.patch
# Move loop index out of OMP firstprivate (c.f. OMP standard).
# Fixme: Does it even make sense to make it explicitly private?
Patch2:		feast-private.patch

BuildRequires:	gcc-gfortran openmpi-devel openblas-devel mpich-devel

%global desc \
The FEAST solver package is a free high-performance numerical library\
for solving the standard or generalized eigenvalue problem, and\
obtaining all the eigenvalues and eigenvectors within a given search\
interval. It is based on an innovative fast and stable numerical\
algorithm -- named the FEAST algorithm -- which deviates fundamentally\
from the traditional Krylov subspace iteration based techniques\
(Arnoldi and Lanczos algorithms) or other Davidson-Jacobi\
techniques. The FEAST algorithm takes its inspiration from the\
density-matrix representation and contour integration technique in\
quantum mechanics. It is free from explicit orthogonalization\
procedures, and its main computational tasks consist of solving very\
few inner independent linear systems with multiple right-hand sides\
and one reduced eigenvalue problem orders of magnitude smaller than\
the original one. The FEAST algorithm combines simplicity and\
efficiency and offers many important capabilities for achieving high\
performance, robustness, accuracy, and scalability on parallel\
architectures.\
\
This general purpose FEAST solver package includes both reverse\
communication interfaces and ready to use predefined interfaces for\
dense, banded and sparse systems. It includes double and single\
precision arithmetic, and all the interfaces are compatible with\
Fortran (77,90) and C. FEAST is both a comprehensive library package,\
and an easy to use software. This solver is expected to significantly\
augment numerical performances and capabilities in large-scale modern\
applications.

%description
%desc

This is the serial version

%package	devel
Summary:	Development files for %{name}
Requires:	%{name}%{?_isa} = %{version}-%{release}

%description	devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}.

%package openmpi
Summary:	Eigenvalue/eigenvector solver - openmpi
Requires:	openmpi%{?_isa}

%description openmpi
%desc

%package	openmpi-devel
Summary:	Development files for %{name}
Requires:	%{name}-openmpi%{?_isa} = %{version}-%{release}, openmpi-devel

%description	openmpi-devel
The %{name}-openmpi-devel package contains libraries and header files for
developing applications that use %{name}-openmpi.

This is the openmpi version.

%package mpich
Summary:	Eigenvalue/eigenvector solver - mpich
Requires:	mpich%{?_isa}

%description mpich
%desc

%package	mpich-devel
Summary:	Development files for %{name}
Requires:	%{name}-mpich%{?_isa} = %{version}-%{release}, mpich-devel

%description	mpich-devel
The %{name}-mpich-devel package contains libraries and header files for
developing applications that use %{name}-mpich.

This is the mpich version.

%prep
%setup -q -n FEAST
cd %version
%patch1 -p2
%patch2 -p2


%build
cd %version/src
gfortran --version
# default is ifort-specific interface (OPTION=2)
# Fixme: Should MPI version have -fopenmp?
%global flags F90=gfortran CFLAGS="%optflags -O3 -fPIC $CFLAGS -fopenmp" \\\
F90FLAGS="%optflags -O3 -ffree-line-length-none -ffixed-line-length-none -cpp -fPIC $FFLAGS -fopenmp" \\\
OPTION=1
make %{?_smp_mflags} ARCH=%_arch LIB=feast %flags shared
rm -f *.o
%_openmpi_load
make %{?_smp_mflags} ARCH=ompi-%_arch LIB=pfeast MPI=mpif90 %flags shared
%_openmpi_unload
rm -f *.o
%_mpich_load
make %{?_smp_mflags} ARCH=mpich-%_arch LIB=pfeast MPI=mpif90 %flags shared
%_mpich_unload


%install
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT%_libdir $RPM_BUILD_ROOT%_includedir
cd %version
cp -a lib/%_arch/*.so* $RPM_BUILD_ROOT%_libdir
install -m 644 include/*.h $RPM_BUILD_ROOT%_includedir
%_openmpi_load
mkdir -p $RPM_BUILD_ROOT$MPI_LIB $RPM_BUILD_ROOT$MPI_INCLUDE
cp -a lib/ompi-%_arch/*.so* $RPM_BUILD_ROOT$MPI_LIB
install -m 644 include/*.h $RPM_BUILD_ROOT$MPI_INCLUDE
%_openmpi_unload
%_mpich_load
mkdir -p $RPM_BUILD_ROOT$MPI_LIB $RPM_BUILD_ROOT$MPI_INCLUDE
cp -a lib/mpich-%_arch/*.so* $RPM_BUILD_ROOT$MPI_LIB
install -m 644 include/*.h $RPM_BUILD_ROOT$MPI_INCLUDE
%_mpich_unload


%clean
rm -rf $RPM_BUILD_ROOT

%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig

%files
%doc %version/LICENSE %version/README
%_libdir/libfeast*.so.*

%files devel
%_libdir/libfeast*.so
%_includedir/*.h

%files openmpi
%doc %version/LICENSE %version/README
%_libdir/openmpi/lib/libpfeast*.so.*

%files openmpi-devel
%_libdir/openmpi/lib/libpfeast*.so
%_includedir/openmpi-%_arch/*

%files mpich
%doc %version/LICENSE %version/README
%_libdir/mpich/lib/libpfeast*.so.*

%files mpich-devel
%_libdir/mpich/lib/libpfeast*.so
%_includedir/mpich-%_arch/*


%changelog
* Thu Nov  5 2015 Dave Love <loveshack@fedoraproject.org> - 3.0-1
- New version
- Patch for OMP error

* Thu Nov  5 2015 Dave Love <loveshack@fedoraproject.org> - 2.1-2
- Actually build against openmpi
- Link against openblas
- Use OPTION 1 for C interface (but links againt libgfortran implicitly)
- Ship all headers, but not mpi ones in non-mpi package
- mpich version

* Mon Jan  5 2015 Dave Love <d.love@liverpool.ac.uk> - 2.1-1
- Initial packaging
